/***************************************************************************
 *   Copyright (C) 2007 by Dr. Feynman                                     *
 *   dr@feynman.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#ifdef WIN32
#include <winsock2.h>
#include "feynman_win32.h"
#else
#include <sys/socket.h>
#include <poll.h>
#endif
#include <assert.h>
#include <stdarg.h>
#include <errno.h>
#include "feynman.h"

//#define DEBUG_DUMP

static int logmode = FEYNMAN_LOGMODE_ALL;

void feynman_compile_time()
{
    feynman_logging(FEYNMAN_LOGMODE_ALL, "libfeynman: Compiled on %s at %s\n", __DATE__, __TIME__);
}

void dump_array(unsigned char *buf, int len)
{
#ifdef DEBUG_DUMP
    int i;

    if(len <= 0)
    {
        return;
    }
    printf("buffer: ");
    for(i=0; i<len; i++)
    {
        printf("%02X ", buf[i]);
    }
    printf("\n");
#endif
}

#ifdef HAVE_SSL
static int safe_SSL_read(SSL *ssl, unsigned char *buf, int size)
{
    int rc;
    int i = 0;
    do
    {
        rc = SSL_read(ssl, buf+i, size-i);
        if(rc < 1)
        {
            int errnum = SSL_get_error(ssl, rc);
            switch(errnum)
            {
                case SSL_ERROR_WANT_READ:
                    if(i != 0) return -1;
                    else return 0;
                    break;
                default:
                    return -1;
                    break;
            }
        }
        else
        {
            i += rc;
        }
    } while (i < size);

    return size;
}

//not corrent
static int safe_SSL_write(SSL *ssl, unsigned char *buf, int size)
{
    int rc;
    int i = 0;
    do
    {
        rc = SSL_write(ssl, buf+i, size-i);
        if(rc < 1)
        {
            int errnum = SSL_get_error(ssl, rc);
            switch(errnum)
            {
                case SSL_ERROR_WANT_READ:
                    if(i != 0) return -1;
                    else return 0;
                    break;
                default:
                    return -1;
                    break;
            }
        }
        else
        {
            i += rc;
        }
    } while (i < size);

    return size;
}

#endif

static int pm_write(t_feynman_handle handle, unsigned char *buf, int len, int tmout)
{
    int i;

    int ret;
    struct pollfd pfd;

    for (i = 0; i < len;)
    {
        pfd.fd = handle.fd;
        pfd.events = POLLOUT | POLLERR;
        pfd.revents = 0;
        /* poll timeout in milliseconds */
        if((ret = poll(&pfd, 1, tmout)) != 1)
        {
            return -1;
        }
        if((pfd.revents & POLLOUT) == POLLOUT)
        {
#ifdef HAVE_SSL
            if(handle.ssl)
            {
                if((ret = safe_SSL_write(handle.ssl, buf+i, len-i)) < 0)
                {
                    return -2;
                }
            }
            else
#endif
            if((ret = write(handle.fd, (char *)buf+i, len-i)) < 0)
            {
                return -2;
            }
            i += ret;
        }
        else
        {
            return -3;
        }
    }
    return i;
}

static int pm_read(t_feynman_handle handle, unsigned char *buf, int len, int tmout)
{
    int i;
    int ret;
    struct pollfd pfd;

#ifdef HAVE_SSL
    if(handle.ssl)
    {
        if((ret = safe_SSL_read(handle.ssl, buf, len)) < 0)
        {
            return -2;
        }
        return ret;
    }
#endif

    for (i = 0; i < len;)
    {
        pfd.fd = handle.fd;
        pfd.events = POLLIN | POLLERR;
        pfd.revents = 0;
        /* poll timeout in milliseconds */
        if((ret = poll (&pfd, 1, tmout)) != 1)
        {
            return -1;
        }

        if((pfd.revents & POLLIN) == POLLIN)
        {
            if((ret = read(handle.fd, (char *)buf+i, len)) < 0)
            {
                return -2;
            }
            i += ret;
        }
        else
        {
            return -3;
        }
    }
    return i;
}

//TODO: Add sanity check marker
int feynman_send_network_message(t_feynman_handle handle, unsigned char *buf, int len)
{
    unsigned char send_buf[FEYNMAN_BUF_SIZE];

    if(len > FEYNMAN_BUF_SIZE+2)
        return FEYNMAN_ERROR;

    send_buf[0] = (len >> 8) & 0xff;
    send_buf[1] = (len >> 0) & 0xff;

    memcpy(send_buf+2, buf, len);

    if(pm_write(handle, send_buf, len+2, 1000) != len+2) return FEYNMAN_ERROR;

    dump_array(send_buf, len+2);
 //   usleep(10000);
    return FEYNMAN_STATUS_OK;
}

//TODO: Add sanity check marker
int feynman_read_network_message(t_feynman_handle handle, unsigned char *buf)
{
    unsigned char read_buf[FEYNMAN_BUF_SIZE];
    int len;

    if((len = pm_read(handle, read_buf, 2, 10000)) < 0) return FEYNMAN_ERROR;

    if(len == 0) return 0;
    if(len == 1) return -1;

    len = (read_buf[0] << 8 ) | (read_buf[1] << 0 );

    if(len > FEYNMAN_BUF_SIZE) return FEYNMAN_ERROR;
    if(pm_read(handle, read_buf+2, len, 10000) != len) return FEYNMAN_ERROR;

    dump_array(read_buf, len+2);

    memcpy(buf, read_buf+2, len);
    return len;
}

void feynman_get_time_diff(struct timeval * tstart, struct timeval * tend)
{
    if (tstart->tv_usec > tend->tv_usec)
    {
        tend->tv_sec = tend->tv_sec - tstart->tv_sec - 1;
        tend->tv_usec = 1000000L - tstart->tv_usec + tend->tv_usec;
    }
    else
    {
        tend->tv_sec = tend->tv_sec - tstart->tv_sec;
        tend->tv_usec = tend->tv_usec - tstart->tv_usec;
    }
}

void feynman_logging(int mode, char *fmt,...)
{
    if(mode & logmode)
    {
        va_list params;
        struct tm times;
        time_t tt;
        char buf[1024];

        tt = time (0);
        memcpy (&times, localtime (&tt), sizeof (struct tm));
        sprintf (buf, "[ %02d:%02d:%02d ] ", times.tm_hour, times.tm_min, times.tm_sec);

        va_start(params, fmt);
        vsnprintf (buf + 13, 1024 - 13 - 1, fmt, params);
        va_end(params);

        printf("%s", buf);

        fflush(stdout);
    }
}

void feynman_logging_set_mode(int mode)
{
    logmode = mode;
}
