/***************************************************************************
 *   Copyright (C) 2007 by Dr. Feynman                                     *
 *   dr@feynman.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "../src/feynman.h"
#include "../src/feynman_win32.h"

char *certificate = "test.pem";

typedef struct 
{
    int a;
    int b;
    char c;
}test_t;

int card_ecm_func(void *args, t_feynman_ecm *ecm)
{
    unsigned char dcw[16];

    printf("[ Test server ] Sending ecm to card\n");
    memset(dcw, ecm->session, 16);

    feynman_set_ecm_dcw(ecm, dcw, 16);
    //usleep(400000);
    return FEYNMAN_ECMSTATUS_DECRYPT_OK;
}

int card_emm_func(void *args, t_feynman_emm *emm)
{
    printf("[ Test server ] Sending emm to card\n");
    return FEYNMAN_STATUS_OK;
}

int main(int argc, char *argv[])
{
    t_feynman_client *client = NULL;
    t_feynman_card *card = NULL;

    t_feynman_local_server *server = feynman_add_local_server(NULL);
    feynman_set_local_server_ecm_callback(server, card_ecm_func);
    feynman_set_local_server_emm_callback(server, card_emm_func);
    feynman_set_local_server_port(server, 55555);

    client = feynman_add_client(server);
    feynman_set_client_username(client, "test1");
    feynman_set_client_password(client, "password");

    int access_mode = FEYNMAN_ACCESS_MODE_ACCESSOK;
    access_mode |= FEYNMAN_ACCESS_MODE_ECM;
    access_mode |= FEYNMAN_ACCESS_MODE_EMM;
    access_mode |= FEYNMAN_ACCESS_MODE_GA;
    access_mode |= FEYNMAN_ACCESS_MODE_SA;
    access_mode |= FEYNMAN_ACCESS_MODE_UA;

    feynman_set_client_access_mode(client, access_mode);

    client = feynman_add_client(server);
    feynman_set_client_username(client, "test2");
    feynman_set_client_password(client, "password");
    feynman_set_client_access_mode(client, access_mode);

    feynman_set_client_access_mode(client, access_mode);

    client = feynman_add_client(server);
    feynman_set_client_username(client, "test3");
    feynman_set_client_password(client, "password");
    feynman_set_client_access_mode(client, access_mode);

    int i, j;

    for(i=1; i<15; i++)
    {
        card = feynman_add_local_card(server, i);
        feynman_set_local_card_caid(card, 0x0b00);
        feynman_set_local_card_provid(card, 0x00000000);
        feynman_set_local_card_ga(card, 0x1234);
        feynman_set_local_card_sa(card, 0x123456);
        feynman_set_local_card_ua(card, 0x12345600 | i);

        for(j=1; j<20; j++)
        {
          //  feynman_add_sid(card, j, FEYNMAN_SIDMODE_ALLOW);
        }


    }
#ifdef HAVE_SSL
    feynman_start_local_server_ssl(server, certificate, certificate);
#else
    feynman_start_local_server(server);
#endif


    while(1)
    {
        usleep(100000);
    }

    return EXIT_SUCCESS;
}
